package com.yunzhi.shigongli.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.shigongli.common.*;
import com.yunzhi.shigongli.entity.*;
import com.yunzhi.shigongli.service.*;
import com.yunzhi.shigongli.vo.ResourceFieldVO;
import com.yunzhi.shigongli.vo.ShopListVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;

/**
 * <p>
    * 商铺 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-08-17
 */

@Api(tags = "商铺")
@RestController
@RequestMapping("/")
public class TaShopController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShopController.class);

    @Autowired
    public ITaShopService iTaShopService;


    @Autowired
    public ITaImageService shopImagesService;

    @Autowired
    public ITaTargetTypeService taTargetTypeService;

    @Autowired
    public ITaPvService iTaPvService;

    @Autowired
    public ITaResourceService taResourceService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/taShop",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taShopList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                   @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                   @ApiParam("店铺名称") @RequestParam(value = "shopName", required = false) String shopName,
                                   @ApiParam("手机号") @RequestParam(value = "phone", required = false) String phone) throws Exception{

		    IPage<ShopListVO> pg = new Page<>(pageNum, pageSize);
            IPage<ShopListVO> result = iTaShopService.getPageListBy(pg, shopName, phone);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param taShop 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taShop",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taShopAdd(@ApiParam("保存内容") @RequestBody TaShop taShop) throws Exception {

        try {
            iTaShopService.createNew(taShop);
            shopImagesService.updateImages(Constants.TARGET_SHOP, taShop.getShopId(), taShop.getImageList());
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseBean.error(e.getMessage(), ResponseBean.ERROR_UNAVAILABLE);
        }

        return ResponseBean.success(taShop);
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taShop/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean taShopDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        if(iTaShopService.removeLogicById(id)){
            taResourceService.removeBy(Constants.TARGET_SHOP, id);
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taShop 实体对象
     * @return
     */
    @RequestMapping(value="/admin/taShop/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taShopUpdate(@ApiParam("对象ID") @PathVariable String id,
                                        @ApiParam("更新内容") @RequestBody TaShop taShop) throws Exception{

        if (iTaShopService.updateById(taShop)){
            shopImagesService.updateImages(Constants.TARGET_SHOP, taShop.getShopId(), taShop.getImageList());
            taTargetTypeService.updateTypes(Constants.TARGET_SHOP, taShop.getShopId(), taShop.getTypeList());

            ResourceFieldVO resourceVO = iTaShopService.copyProperties(taShop);
            TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
            BeanUtils.copyProperties(resourceVO, taResource);

            taResourceService.updateById(taResource);
            return ResponseBean.success(iTaShopService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


    /**
     * 商铺营业
     * @param id  实体ID
     * @return
     */
    @RequestMapping(value="/admin/taShop/{id}/on",method= RequestMethod.PUT)
    @ApiOperation(value="商铺营业", notes = "商铺营业", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean publish(@ApiParam("对象ID") @PathVariable String id) throws Exception {

        TaShop taShop = iTaShopService.getById(id);
        taShop.setStatus(Constants.STATUS_NORMAL);

        if (iTaShopService.updateById(taShop)){
            TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
            taResource.setStatus(taShop.getStatus());
            taResourceService.updateById(taResource);

            return ResponseBean.success(taShop);
        }else {
            return ResponseBean.error("发布失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 商铺关闭
     * @param id  实体ID
     * @return
     */
    @RequestMapping(value="/admin/taShop/{id}/off",method= RequestMethod.PUT)
    @ApiOperation(value="商铺关闭", notes = "商铺关闭", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean unPublish(@ApiParam("对象ID") @PathVariable String id) throws Exception {

        TaShop taShop = iTaShopService.getById(id);
        taShop.setStatus(Constants.STATUS_READY);

        if (iTaShopService.updateById(taShop)){
            TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
            taResource.setStatus(taShop.getStatus());
            taResourceService.updateById(taResource);

            return ResponseBean.success(taShop);
        }else {
            return ResponseBean.error("取消发布失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }



    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/taShop/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taShopGet(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaShop taShop = iTaShopService.getById(id);
        if (Constants.STATUS_DELETED.equals(taShop.getStatus())) {
            return ResponseBean.error("当前商铺信息不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
        taShop.setPvNum(taResource.getPvNum());
        taShop.setUvNum(taResource.getUvNum());
        taShop.setLikeNum(taResource.getLikeNum());
        taShop.setImageList(shopImagesService.getListBy(Constants.TARGET_SHOP, id));
        taShop.setTypeList(taTargetTypeService.getListBy(Constants.TARGET_SHOP, id));

        return ResponseBean.success(taShop);
    }

    /**
     * 微信端详情
     * @param id
     * @return
     * @throws Exception
     */
    @ApiOperation(value="微信端详情", notes = "微信端详情", httpMethod = "GET", response = ResponseBean.class)
    @GetMapping("/wx/shop/{id}")
    public ResponseBean getWxDetail(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        TaShop taShop = iTaShopService.getById(id);
        if (Constants.STATUS_DELETED.equals(taShop.getStatus())) {
            return ResponseBean.error("当前商铺信息不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaResource taResource = taResourceService.getByTarget(Constants.TARGET_SHOP, id);
        taShop.setPvNum(taResource.getPvNum());
        taShop.setUvNum(taResource.getUvNum());
        taShop.setLikeNum(taResource.getLikeNum());
        taShop.setImageList(shopImagesService.getListBy(Constants.TARGET_SHOP, id));

        // 统计 PV, UV
        try {
            taResourceService.recordPerson(Constants.TARGET_SHOP, id, getCurrentPerson());
        } catch (Exception e) {
            e.printStackTrace();
        }

        return ResponseBean.success(taShop);
    }

}
